var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, NgModule, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux/index';
var ReduxRegistryReducerItem = (function () {
    function ReduxRegistryReducerItem() {
    }
    return ReduxRegistryReducerItem;
}());
var ReduxRegistry = (function () {
    function ReduxRegistry() {
    }
    /**
     * @param {?} store
     * @return {?}
     */
    ReduxRegistry.registerStore = function (store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    };
    /**
     * @param {?} stateName
     * @param {?} type
     * @param {?} reducer
     * @return {?}
     */
    ReduxRegistry.registerReducer = function (stateName, type, reducer) {
        ReduxRegistry._reducers[type] = ReduxRegistry._reducers[type] || [];
        ReduxRegistry._reducers[type].push({ stateName: stateName, reducer: reducer });
    };
    /**
     * @param {?} stateName
     * @param {?} module
     * @return {?}
     */
    ReduxRegistry.registerModule = function (stateName, module) {
        ReduxRegistry.getStore().then(function (store) {
            var /** @type {?} */ initialState = {};
            if (module.onReduxInit) {
                initialState = module.onReduxInit() || {};
            }
            store.dispatch({
                payload: {
                    initialState: initialState,
                    stateName: stateName,
                },
                type: ReduxRegistry.ACTION_REGISTER_MODULE,
            });
        });
    };
    /**
     * @param {?} type
     * @return {?}
     */
    ReduxRegistry.getReducerItemsByType = function (type) {
        return ReduxRegistry._reducers[type] || [];
    };
    /**
     * @return {?}
     */
    ReduxRegistry.getStore = function () {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    };
    return ReduxRegistry;
}());
ReduxRegistry.ACTION_REGISTER_MODULE = ReduxRegistry.name + "://registerModule";
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = {};
/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @return {?}
 */
function selectByState(state, selector) {
    var /** @type {?} */ selectorArray = selector.split('/');
    if (selectorArray[0] !== '') {
        throw new Error('Relative selectors are not supported yet!');
    }
    return selectorArray
        .filter(function (propertyKey) { return propertyKey !== ''; })
        .reduce(function (previousValue, propertyKey) {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @return {?}
 */
function select(selector) {
    var /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(function (store) {
        subject.next(selectByState(store.getState(), selector));
        store.subscribe(function () { return subject.next(selectByState(store.getState(), selector)); });
    });
    return subject.asObservable();
}
/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(function (store) { return store.dispatch(action); });
}
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return function (target, propertyKey, descriptor) {
        var /** @type {?} */ method = descriptor.value;
        var /** @type {?} */ type = customType || [/** @type {?} */ (target['name']) || target.constructor.name,
            propertyKey,
        ].join(target['name'] ? '/static/' : '/');
        var /** @type {?} */ proxyFunction = function () {
            var /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then(function (p) { return dispatch({ payload: p, type: type }); });
            }
            else {
                dispatch({ payload: payload, type: type });
            }
            return payload;
        };
        proxyFunction['__@ReduxAction'] = { type: type };
        descriptor.value = proxyFunction;
    };
}
/**
 * @template S, P
 * @param {?} typeArray
 * @return {?}
 */
function ReduxReducer(typeArray) {
    var /** @type {?} */ types = (Array.isArray(typeArray) ? typeArray : [typeArray]).map(function (t) {
        if (typeof t === 'function') {
            if (!t['__@ReduxAction'] || !t['__@ReduxAction'].type) {
                throw new Error('Unable to resolve action type. Make sure you\'ve decorated the target by @ReduxAction');
            }
            return t['__@ReduxAction'].type;
        }
        return t;
    });
    return function (target, propertyKey) {
        if (typeof target === 'object') {
            target[propertyKey].bind(target);
        }
        if (!target['__@ReduxReducer']) {
            target['__@ReduxReducer'] = [];
        }
        types.forEach(function (type) {
            target['__@ReduxReducer'].push({
                actionType: type,
                reducerFunction: target[propertyKey].bind(target),
            });
        });
        return target[propertyKey];
    };
}
/**
 * @param {?=} config
 * @return {?}
 */
function Redux(config) {
    return function (constructor) {
        return (function (_super) {
            __extends(class_1, _super);
            function class_1() {
                var _this = _super.apply(this, arguments) || this;
                _this.redux = (function () {
                    var /** @type {?} */ mergedConfig = Object.assign({
                        reducers: [],
                        stateName: constructor.name,
                    }, config || {});
                    var stateName = mergedConfig.stateName, reducers = mergedConfig.reducers;
                    // constructor[ '__@Redux' ] = mergedConfig;
                    ReduxRegistry.registerModule(stateName, _this);
                    reducers.reduce(function (previousValue, currentValue) {
                        return previousValue
                            .concat(currentValue['__@ReduxReducer'] || [])
                            .concat(currentValue['prototype']['__@ReduxReducer'] || []);
                    }, []).forEach(function (r) { return ReduxRegistry.registerReducer(stateName, r.actionType, r.reducerFunction); });
                    return config;
                })();
                return _this;
            }
            return class_1;
        }(constructor));
    };
}
/**
 * @param {?} selector
 * @return {?}
 */
function ReduxSelect(selector) {
    return function (target, propertyKey) {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector),
        });
    };
}
var ReduxSelectPipe = (function () {
    function ReduxSelectPipe() {
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    ReduxSelectPipe.prototype.transform = function (selector) {
        return select(selector);
    };
    return ReduxSelectPipe;
}());
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = function () { return []; };
var rootReducer = function (state, action) {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        var _a = action.payload, stateName = _a.stateName, initialState = _a.initialState;
        return Object.assign({}, state, (_b = {},
            _b[stateName] = initialState,
            _b));
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce(function (stateToReduce, reducerItem) {
        var stateName = reducerItem.stateName, reducer = reducerItem.reducer;
        return Object.assign({}, stateToReduce, (_a = {},
            _a[stateName] = reducer(stateToReduce[stateName], action),
            _a));
        var _a;
    }, state);
    var _b;
};
var REDUX_MODULE_CONFIG = new InjectionToken('REDUX_MODULE_CONFIG');
var IS_ROOT_MODULE = new InjectionToken('IS_ROOT_MODULE');
var ReduxModule = (function () {
    /**
     * @param {?=} isRootModule
     * @param {?=} config
     */
    function ReduxModule(isRootModule, config) {
        if (isRootModule === void 0) { isRootModule = false; }
        if (config === void 0) { config = null; }
        if (isRootModule) {
            config = Object.assign({
                store: null,
            }, config || {});
            ReduxRegistry.registerStore(config.store || createStore(rootReducer, {}));
        }
    }
    /**
     * @return {?}
     */
    ReduxModule.forChild = function () {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: IS_ROOT_MODULE, useValue: false },
            ],
        };
    };
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxModule.forRoot = function (config) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: REDUX_MODULE_CONFIG, useValue: config },
                { provide: IS_ROOT_MODULE, useValue: true },
            ],
        };
    };
    return ReduxModule;
}());
ReduxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ReduxModule.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [IS_ROOT_MODULE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_MODULE_CONFIG,] },] },
]; };
/**
 * Generated bundle index. Do not edit.
 */
export { selectByState, select, ReduxAction, ReduxReducer, Redux, ReduxSelect, ReduxSelectPipe, REDUX_MODULE_CONFIG, IS_ROOT_MODULE, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.es5.js.map
